@@ -21,7 +21,6 @@
 
 typedef struct _ProcConfig ProcConfig;
 typedef struct _PrettyTable PrettyTable;
-typedef struct _LoadGraph LoadGraph;
 typedef struct _ProcInfo ProcInfo;
 typedef struct _ProcData ProcData;
 
@@ -34,6 +33,8 @@ typedef struct _ProcData ProcData;
 #include "infoview.h"
 #include "smooth_refresh.h"
 
+#include "load-graph.h"
+
 enum
 {
 	ALL_PROCESSES,
@@ -68,6 +69,8 @@ struct _ProcConfig
 	GdkColor	cpu_color[GLIBTOP_NCPU];
 	GdkColor	mem_color;
 	GdkColor	swap_color;
+	GdkColor	net_in_color;
+	GdkColor	net_out_color;
 	GdkColor	bg_color;
 	GdkColor	frame_color;
 	gint 		num_cpus;
@@ -79,42 +82,6 @@ struct _PrettyTable {
 	gchar *datadir;
 };
 
-struct _LoadGraph {
-    
-    guint n;
-    gint type;
-    guint speed;
-    guint draw_width, draw_height;
-    guint num_points;
-    guint num_cpus;
-
-    guint allocated;
-
-    GdkColor *colors;
-    gfloat **data;
-    guint data_size;
-    guint *pos;
-
-    gint colors_allocated;
-    GtkWidget *main_widget;
-    GtkWidget *disp;
-    GtkWidget *cpu_labels[GLIBTOP_NCPU];
-    GtkWidget *memused_label;
-    GtkWidget *memtotal_label;
-    GtkWidget *mempercent_label;
-    GtkWidget *swapused_label;
-    GtkWidget *swaptotal_label;
-    GtkWidget *swappercent_label;
-    GdkPixmap *pixmap;
-    GdkGC *gc;
-    int timer_index;
-    
-    gboolean draw;
-
-    guint64 cpu_time [GLIBTOP_NCPU] [NCPUSTATES];
-    guint64 cpu_last [GLIBTOP_NCPU] [NCPUSTATES];
-    gboolean cpu_initialized;       
-};
 
 
 struct _ProcInfo
@@ -168,11 +135,13 @@ struct _ProcData
 	ProcConfig	config;
 	LoadGraph	*cpu_graph;
 	LoadGraph	*mem_graph;
+	LoadGraph	*net_graph;
 	gint		cpu_label_fixed_width;
+	gint		net_label_fixed_width;
 	ProcInfo	*selected_process;
 	GtkTreeSelection *selection;
-	gint		timeout;
-	gint		disk_timeout;
+	guint		timeout;
+	guint		disk_timeout;
 
 /*
    'info' is GList, which has very slow lookup. On each update, glibtop
@@ -229,4 +198,24 @@ void		procman_save_config (ProcData *data) G_GNUC_INTERNAL;
 void		procman_save_tree_state (GConfClient *client, GtkWidget *tree, const gchar *prefix) G_GNUC_INTERNAL;
 gboolean	procman_get_tree_state (GConfClient *client, GtkWidget *tree, const gchar *prefix) G_GNUC_INTERNAL;
 
+
+
+
+
+struct ReniceArgs
+{
+	ProcData *procdata;
+	int nice_value;
+};
+
+
+struct KillArgs
+{
+	ProcData *procdata;
+	int signal;
+};
+
+
+
+
 #endif /* _PROCMAN_PROCMAN_H_ */
