@@ -49,11 +49,7 @@
 #include "infoview.h"
 #include "interface.h"
 #include "favorites.h"
-
-
-#ifdef ENABLE_SELINUX
-#include <selinux/selinux.h>
-#endif
+#include "selinux.h"
 
 
 static guint64 total_time = 1;
@@ -101,43 +97,6 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 
 
 
-static gboolean
-can_show_security_context_column (void)
-{
-#ifdef ENABLE_SELINUX
-	switch (is_selinux_enabled()) {
-	case 1: /* We're running on an SELinux kernel */ 
-		return TRUE;
-
-	default:
-	case -1: 
-		/* Error; hide the security context column */
-
-	case 0: 
-		/* We're not running on an SELinux kernel: hide the security context column */
-		return FALSE;
-	}
-#else
-	/* Support disabled; hide the security context column */
-	return FALSE;
-#endif
-}
-
-static void
-get_process_selinux_context (ProcInfo *info)
-{
-#ifdef ENABLE_SELINUX
-	/* Directly grab the SELinux security context: */
-	security_context_t con;
-
-	if (!getpidcon (info->pid, &con)) {
-		info->security_context = g_strdup (con);
-		freecon (con);
-	}
-#endif
-}
-
-
 static void
 set_proctree_reorderable(ProcData *procdata)
 {
@@ -155,6 +114,90 @@ set_proctree_reorderable(ProcData *procdata)
 }
 
 
+static void
+cb_columns_changed(GtkTreeView *treeview, gpointer user_data)
+{
+	ProcData * const procdata = user_data;
+
+	procman_save_tree_state(procdata->client,
+				GTK_WIDGET(treeview),
+				"/apps/procman/proctree");
+}
+
+
+static GtkTreeViewColumn*
+my_gtk_tree_view_get_column_with_sort_column_id(GtkTreeView *treeview, int id)
+{
+	GList *columns, *it;
+	GtkTreeViewColumn *col = NULL;
+
+	columns = gtk_tree_view_get_columns(treeview);
+
+	for(it = columns; it; it = it->next)
+	{
+		if(gtk_tree_view_column_get_sort_column_id(it->data) == id)
+		{
+			col = it->data;
+			break;
+		}
+	}
+
+	g_list_free(columns);
+
+	return col;
+}
+
+
+void
+proctable_set_columns_order(GtkTreeView *treeview, GSList *order)
+{
+	GtkTreeViewColumn* last = NULL;
+	GSList *it;
+
+	for(it = order; it; it = it->next)
+	{
+		int id;
+		GtkTreeViewColumn *cur;
+
+		id = GPOINTER_TO_INT(it->data);
+
+		g_assert(id >= 0 && id < NUM_COLUMNS);
+
+		cur = my_gtk_tree_view_get_column_with_sort_column_id(treeview, id);
+
+		if(cur && cur != last)
+		{
+			gtk_tree_view_move_column_after(treeview, cur, last);
+			last = cur;
+		}
+	}
+}
+
+
+
+GSList*
+proctable_get_columns_order(GtkTreeView *treeview)
+{
+	GList *columns, *col;
+	GSList *order = NULL;
+
+	columns = gtk_tree_view_get_columns(treeview);
+
+	for(col = columns; col; col = col->next)
+	{
+		int id;
+
+		id = gtk_tree_view_column_get_sort_column_id(col->data);
+		order = g_slist_prepend(order, GINT_TO_POINTER(id));
+	}
+
+	g_list_free(columns);
+
+	order = g_slist_reverse(order);
+
+	return order;
+}
+
 
 static gboolean
 search_equal_func(GtkTreeModel *model,
@@ -197,7 +240,7 @@ proctable_new (ProcData * const procdata)
 		N_("User"),
 		N_("Status"),
 		N_("Memory"),
-		N_("VM Size"),
+		N_("Virtual Memory"),
 		N_("Resident Memory"),
 		N_("Shared Memory"),
 		N_("RSS Memory"),
@@ -273,7 +316,7 @@ proctable_new (ProcData * const procdata)
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (proctree), column);
 
 
-	for (i = 1; i < NUM_COLUMNS - 2; i++) {
+	for (i = COL_USER; i <= COL_ARGS; i++) {
 		cell_renderer = gtk_cell_renderer_text_new ();
 
 		switch(i)
@@ -308,7 +351,7 @@ proctable_new (ProcData * const procdata)
 	gtk_container_add (GTK_CONTAINER (scrolled), proctree);
 
 
-	for(i = 0; i< NUM_COLUMNS - 2; i++)
+	for(i = COL_NAME; i <= COL_ARGS; i++)
 	{
 		switch(i)
 		{
@@ -349,6 +392,9 @@ proctable_new (ProcData * const procdata)
 	g_signal_connect (G_OBJECT (proctree), "button_press_event",
 			  G_CALLBACK (cb_tree_button_pressed), procdata);
 
+	g_signal_connect (G_OBJECT(proctree), "columns-changed",
+			 G_CALLBACK(cb_columns_changed), procdata);
+
 	return scrolled;
 }
 
@@ -358,10 +404,14 @@ proctable_free_info (ProcData *procdata, ProcInfo *info)
 {
 	g_return_if_fail(info != NULL);
 
+	if(info->pixbuf) {
+		g_object_unref (info->pixbuf);
+	}
+
 	g_free (info->name);
 	g_free (info->arguments);
 	g_free (info->security_context);
-	g_list_free (info->children);
+	g_slist_free (info->children);
 	g_mem_chunk_free (procdata->procinfo_allocator, info);
 }
 
@@ -647,7 +697,6 @@ void
 remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 {
 	GtkTreeModel *model;
-	GtkTreeIter iter;
 
 	g_return_if_fail (info);
 
@@ -655,12 +704,11 @@ remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 		return;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
-	iter = info->node;
 
 	if (procdata->selected_process == info)
 		procdata->selected_process = NULL;
 
-	gtk_tree_store_remove (GTK_TREE_STORE (model), &iter);
+	gtk_tree_store_remove (GTK_TREE_STORE (model), &info->node);
 
 	info->is_visible = FALSE;
 }
@@ -669,21 +717,21 @@ remove_info_from_tree (ProcInfo *info, ProcData *procdata)
 static void
 remove_info_from_list (ProcInfo *info, ProcData *procdata)
 {
-	GList *child;
+	GSList *child;
 	ProcInfo * const parent = info->parent;
 
 	/* Remove info from parent list */
 	if (parent)
-		parent->children = g_list_remove (parent->children, info);
+		parent->children = g_slist_remove (parent->children, info);
 
 	/* Add any children to parent list */
-	for(child = info->children; child; child = g_list_next (child)) {
+	for(child = info->children; child; child = g_slist_next (child)) {
 		ProcInfo *child_info = child->data;
 		child_info->parent = parent;
 	}
 
 	if(parent) {
-		parent->children = g_list_concat(parent->children,
+		parent->children = g_slist_concat(parent->children,
 						 info->children);
 		info->children = NULL;
 	}
@@ -720,9 +768,6 @@ update_info (ProcData *procdata, ProcInfo *info)
 	}
 
 	if (info->is_visible) {
-		GdkRectangle rect, vis_rect;
-		GtkTreePath *path;
-
 		GtkTreeModel *model;
 		glibtop_proc_uid procuid;
 		glibtop_proc_time proctime;
@@ -741,17 +786,6 @@ update_info (ProcData *procdata, ProcInfo *info)
 
 
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (procdata->tree));
-		path = gtk_tree_model_get_path (model, &info->node);
-		gtk_tree_view_get_cell_area (GTK_TREE_VIEW (procdata->tree),
-					     path, NULL, &rect);
-		gtk_tree_view_get_visible_rect (GTK_TREE_VIEW (procdata->tree),
-						&vis_rect);
-		gtk_tree_path_free (path);
-
-		/* Don't update if row is not visible.
-		   Small performance improvement */
-		if ((rect.y < 0) || (rect.y > vis_rect.height))
-			return;
 
 		update_info_mutable_cols(GTK_TREE_STORE (model), procdata, info);
 	}
@@ -796,7 +830,7 @@ get_info (ProcData *procdata, gint pid)
 
 	info->parent = find_parent (procdata, procuid.ppid);
 	if (info->parent) {
-		info->parent->children = g_list_prepend (info->parent->children, info);
+		info->parent->children = g_slist_prepend (info->parent->children, info);
 		if(g_str_equal(info->name, info->parent->name)
 		   && info->parent->mem == info->mem)
 			info->is_thread = TRUE;
