@@ -57,6 +57,9 @@ void		proctable_update_all (ProcData *data) G_GNUC_INTERNAL;
 void		proctable_clear_tree (ProcData *data) G_GNUC_INTERNAL;
 void		proctable_free_table (ProcData *data) G_GNUC_INTERNAL;
 
+GSList*		proctable_get_columns_order(GtkTreeView *treeview) G_GNUC_INTERNAL;
+void		proctable_set_columns_order(GtkTreeView *treeview, GSList *order) G_GNUC_INTERNAL;
+
 char*		make_loadavg_string(void) G_GNUC_INTERNAL;
 
 #endif /* _PROCMAN_PROCTABLE_H_ */
